local VERSION = tonumber(("$Revision: 42593 $"):match("%d+"))

local Parrot = Parrot
local Parrot_Events = Parrot:NewModule("Events", "Parser-3.0", "AceEvent-2.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-03 04:56:48 -0400 (Tue, 03 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot-Events")

local new, del = Parrot.new, Parrot.del

local combatEvents = {}

local function parserHandler(data, info)
	Parrot_Events:TriggerCombatEvent(data.category, data.name, info.isCrit, info)
end

function Parrot_Events:RegisterCombatEvent(data)
	AceLibrary.argCheck(self, data, 2, "table")
	local category = data.category
	if type(category) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. category must be a %q, got %q."):format("string", type(category)), 2)
	end
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	local defaultTag = data.defaultTag
	if type(defaultTag) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. defaultTag must be a %q, got %q."):format("string", type(defaultTag)), 2)
	end
	local parserEvent = data.parserEvent
	if parserEvent and type(parserEvent) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. parserEvent must be a %q or nil, got %q."):format("table", type(parserEvent)), 2)
	end
	local blizzardEvent = data.blizzardEvent
	if blizzardEvent and type(blizzardEvent) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. blizzardEvent must be a %q or nil, got %q."):format("string", type(blizzardEvent)), 2)
	end
	if parserEvent and blizzardEvent then
		error("Bad argument #2 to `RegisterCombatEvent'. blizzardEvent and parserEvent cannot coexist.", 2)
	end
	local tagTranslations = data.tagTranslations
	if tagTranslations and type(tagTranslations) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. tagTranslations must be a %q or nil, got %q."):format("table", type(tagTranslations)), 2)
	end
	if not combatEvents[category] then
		combatEvents[category] = new()
	end
	combatEvents[category][name] = data
	if blizzardEvent then
		self:RegisterEvent(blizzardEvent, function(...)
			local info = new(...)
			self:TriggerCombatEvent(category, name, true, info)
			info = del(info)
		end)
	elseif parserEvent then
		self:RegisterParserEvent(parserEvent, parserHandler, data)
	end
end

local handler__translation
local handler__info
local function handler(literal)
	local inner = literal:sub(2, -2)
	local value = handler__translation[inner]
	if value then
		if type(value) == "string" then
			return tostring(handler__info[value])
		else
			return tostring(value(handler__info))
		end
	else
		return "[" .. inner:gsub("(%b[])", handler) .. "]"
	end
end

function Parrot_Events:TriggerCombatEvent(kind, name, sticky, info)
	local data = combatEvents[kind]
	if not data then
		return
	end
	data = data[name]
	if not data then
		return
	end
	local text = data.defaultTag
	handler__translation = data.tagTranslations
	if handler__translation then
		handler__info = info
		text = text:gsub("(%b[])", handler)
		handler__translation = nil
		handler__info = nil
	end
	Parrot:ShowMessage(text, kind, sticky)
end


Parrot_Events:RegisterCombatEvent{
	category = "Notification",
	name = "Enter combat",
	localName = L["Enter combat"],
	defaultTag = "+Combat",
	blizzardEvent = "PLAYER_REGEN_DISABLED",
}

Parrot_Events:RegisterCombatEvent{
	category = "Notification",
	name = "Leave combat",
	localName = L["Leave combat"],
	defaultTag = "-Combat",
	blizzardEvent = "PLAYER_REGEN_ENABLED",
}

local damageTypeToColor = {
	["Physical"] = "ffffff",
	["Holy"] = "ffff7f",
	["Fire"] = "ff7f7f",
	["Nature"] = "7fff7f",
	["Frost"] = "7f7fff",
	["Shadow"] = "7f007f",
	["Arcane"] = "ff7fff",
}

local coloredDamageAmount = function(info) return ("|cff%s%s|r"):format(damageTypeToColor[info.damageType], info.amount) end

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Damage",
	localName = L["Melee Damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Misses",
	localName = L["Melee Misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Dodges",
	localName = L["Melee Dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Parries",
	localName = L["Melee Parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Blocks",
	localName = L["Melee Blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Absorbs",
	localName = L["Melee Absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Immunes",
	localName = L["Melee Immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Damage",
	localName = L["Skill Damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	throttle = true,
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Misses",
	localName = L["Ability Misses"],
	defaultTag = "([Skill]) " .. L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Dodges",
	localName = L["Ability Dodges"],
	defaultTag = "([Skill]) " .. L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Parries",
	localName = L["Ability Parries"],
	defaultTag = "([Skill]) " .. L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Blocks",
	localName = L["Ability Blocks"],
	defaultTag = "([Skill]) " .. L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}


Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell Resists",
	localName = L["Spell Resists"],
	defaultTag = "([Skill]) " .. L["Resist!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Absorbs",
	localName = L["Skill Absorbs"],
	defaultTag = "([Skill]) " .. L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Immunes",
	localName = L["Skill Immunes"],
	defaultTag = "([Skill]) " .. L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Reflects",
	localName = L["Skill Reflects"],
	defaultTag = "([Skill]) " .. L["Reflect!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell Interrupts",
	localName = L["Skill Interrupts"],
	defaultTag = "([Skill]) " .. L["Interrupt!"],
	parserEvent = {
		eventType = "Interrupt",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
		Amount = "realAmount",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals over Time",
	localName = L["Heals over Time"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	throttle = true,
}

Parrot_Events:RegisterCombatEvent{
	category = "Incoming",
	name = "Environmental Damage",
	localName = L["Environmental Damage"],
	defaultTag = "-[Amount] [Type]",
	parserEvent = {
		eventType = "Environmental",
		recipientID = "player",
	},
	tagTranslations = {
		Amount = coloredDamageAmount,
		Type = "hazardTypeLocal",
	},
	throttle = true,
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Damage",
	localName = L["Melee Damage"],
	defaultTag = "[Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Misses",
	localName = L["Melee Misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Dodges",
	localName = L["Melee Dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Parries",
	localName = L["Melee Parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Blocks",
	localName = L["Melee Blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Absorbs",
	localName = L["Melee Absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Immunes",
	localName = L["Melee Immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Evades",
	localName = L["Melee Evades"],
	defaultTag = L["Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Damage",
	localName = L["Skill Damage"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	throttle = true,
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Misses",
	localName = L["Ability Misses"],
	defaultTag = L["Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Dodges",
	localName = L["Ability Dodges"],
	defaultTag = L["Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Parries",
	localName = L["Ability Parries"],
	defaultTag = L["Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell Resists",
	localName = L["Spell Resists"],
	defaultTag = L["Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Absorbs",
	localName = L["Skill Absorbs"],
	defaultTag = L["Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Immunes",
	localName = L["Skill Immunes"],
	defaultTag = L["Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Reflects",
	localName = L["Skill Reflects"],
	defaultTag = L["Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell Interrupts",
	localName = L["Skill Interrupts"],
	defaultTag = L["Interrupt!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Interrupt",
		sourceID = "player",
		recipientID_not = "player",
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}


Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Evades",
	localName = L["Skill Evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	}
}

Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
		Amount = "realAmount",
	}
}


Parrot_Events:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals over Time",
	localName = L["Heals over Time"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	throttle = true,
}

